Index: funcs/func_strings.c
===================================================================
--- funcs/func_strings.c	(revision 180528)
+++ funcs/func_strings.c	(working copy)
@@ -162,6 +162,23 @@
 	.read = regex,
 };
 
+static int strecmp(const char *pre, const char *post)
+{
+	int res;
+	for (; *pre && *post; pre++, post++) {
+		if (*pre == '"') {
+			post--;
+			continue;
+		} else if (*pre == '\\') {
+			pre++;
+		}
+		if ((res = strncmp(pre, post, 1))) {
+			return res;
+		}
+	}
+	return strncmp(pre, post, 1);
+}
+
 static int array(struct ast_channel *chan, char *cmd, char *var,
 		 const char *value)
 {
@@ -174,6 +191,15 @@
 	char *value2;
 	int i;
 
+	if (chan) {
+		const char *value3;
+		ast_mutex_lock(&chan->lock);
+		if ((value3 = pbx_builtin_getvar_helper(chan, "~ODBCVALUES~")) && strecmp(value3, value) == 0) {
+			value = ast_strdupa(value3);
+		}
+		ast_mutex_unlock(&chan->lock);
+	}
+
 	value2 = ast_strdupa(value);
 	if (!var || !value2)
 		return -1;
Index: funcs/func_odbc.c
===================================================================
--- funcs/func_odbc.c	(revision 180528)
+++ funcs/func_odbc.c	(working copy)
@@ -227,6 +227,9 @@
 	SQLSMALLINT colcount=0;
 	SQLLEN indicator;
 
+	/* Reset, in case of an error */
+	pbx_builtin_setvar_helper(chan, "~ODBCVALUES~", NULL);
+
 	AST_LIST_LOCK(&queries);
 	AST_LIST_TRAVERSE(&queries, query, list) {
 		if (!strcmp(query->acf->name, cmd)) {
@@ -322,11 +325,11 @@
 		return res1;
 	}
 
+	buflen = strlen(buf);
 	for (x = 0; x < colcount; x++) {
 		int i;
 		char coldata[256];
 
-		buflen = strlen(buf);
 		res = SQLGetData(stmt, x + 1, SQL_CHAR, coldata, sizeof(coldata), &indicator);
 		if (indicator == SQL_NULL_DATA) {
 			coldata[0] = '\0';
@@ -342,25 +345,23 @@
 				ast_autoservice_stop(chan);
 			if (bogus_chan)
 				ast_channel_free(chan);
+			/* Terminate buffer just in case */
+			buf[buflen] = '\0';
 			return -1;
 		}
 
 		/* Copy data, encoding '\' and ',' for the argument parser */
-		for (i = 0; i < sizeof(coldata); i++) {
-			if (escapecommas && (coldata[i] == '\\' || coldata[i] == ',')) {
+		for (i = 0; i < sizeof(coldata) && buflen < len - 1; i++) {
+			if (coldata[i] == '\0')
+				break;
+
+			if (escapecommas && strchr("\",\\|", coldata[i]))
 				buf[buflen++] = '\\';
-			}
+
 			buf[buflen++] = coldata[i];
-
-			if (buflen >= len - 2)
-				break;
-
-			if (coldata[i] == '\0')
-				break;
 		}
 
-		buf[buflen - 1] = ',';
-		buf[buflen] = '\0';
+		buf[buflen++] = ',';
 	}
 	/* Trim trailing comma */
 	buf[buflen - 1] = '\0';
@@ -368,6 +369,10 @@
 	SQLCloseCursor(stmt);
 	SQLFreeHandle(SQL_HANDLE_STMT, stmt);
 	ast_odbc_release_obj(obj);
+
+	/* Pass an unadulterated string to ARRAY, if needed.  This is only needed
+	 * in 1.4, because of the misfeature in Set. */
+	pbx_builtin_setvar_helper(chan, "~ODBCVALUES~", buf);
 	if (chan)
 		ast_autoservice_stop(chan);
 	if (bogus_chan)
